import click
import logging
import os

DEFAULT_LOG_FORMAT = '%(levelname)s: %(message)s'
VERBOSE_LOG_FORMAT = '%(asctime)s %(levelname)s %(name)s: %(message)s'

@click.group()
@click.option('-v', '--verbose', count=True)
@click.version_option()
def main(verbose=0):
    level = logging.INFO
    format = DEFAULT_LOG_FORMAT
    if verbose > 0:
        level = logging.DEBUG
        format = VERBOSE_LOG_FORMAT
    logging.basicConfig(format=format, level=level)

@click.command('s3:push')
@click.option('-b', '--bucket', required=True)
@click.option('-p', '--public', default=False, is_flag=True)
@click.option('--acl')
@click.option('--prefix')
@click.option('--expires', type=int, default=86400)
@click.argument('packages', nargs=-1, required=True)
def s3_push(packages, bucket, acl, prefix, public, expires):
    from .push import push
    from .s3 import S3Target

    if not acl:
        acl = 'public-read' if public else 'private'

    target = S3Target(
        bucket=bucket,
        acl=acl,
        public=public,
        expires=expires,
    )

    url = push(target, packages, prefix=prefix)
    click.echo(url)

@click.command('local:push')
@click.option('--url', required=True)
@click.option('--prefix')
@click.argument('packages', nargs=-1, required=True)
def local_push(packages, url, prefix):
    from .local import LocalTarget
    from .push import push

    # it's surprising when the final segment is chopped off by urljoin
    # so ensure there's always a trailing slash
    if not url.endswith('/'):
        url += '/'

    target = LocalTarget(
        url=url,
        root_dir=os.getcwd(),
    )

    url = push(target, packages, prefix=prefix)
    click.echo(url)

main.add_command(s3_push)
main.add_command(local_push)



import attr
import boto3
import typing

log = __import__('logging').getLogger(__name__)

@attr.s(auto_attribs=True)
class S3Target:
    bucket: str
    acl: str
    public: bool
    expires: int
    client: typing.Any = None

    def __attrs_post_init__(self):
        if self.client is None:
            self.client = boto3.client('s3')

    def put_object(self, path, data, content_type):
        log.info(f'uploading object to s3://{self.bucket}/{path}')
        self.client.put_object(
            Bucket=self.bucket,
            Key=path,
            Body=data,
            ACL=self.acl,
            ContentType=content_type,
        )

        url = self.client.generate_presigned_url(
            'get_object',
            Params=dict(
                Bucket=self.bucket,
                Key=path,
            ),
            ExpiresIn=self.expires,
        )
        if self.public:
            i = url.index('?')
            url = url[:i]
        log.debug(f'object url={url}')
        return url



import importlib.resources
import mimetypes
import os.path

from .apk import is_apk, open_apk
from .ipa import is_ipa, open_ipa
from .templates import render_template

def push(target, package_paths, prefix=None):
    if not prefix:
        prefix = os.path.splitext(os.path.basename(package_paths[0]))[0]

    packages = []
    for package_path in package_paths:
        if is_ipa(package_path):
            with open_ipa(package_path) as ipa:
                ctx = push_ipa(ipa, target, prefix)

        elif is_apk(package_path):
            with open_apk(package_path) as apk:
                ctx = push_apk(apk, target, prefix)

        else:
            raise ValueError('unrecognized package')

        packages.append(ctx)

    html_tmpl = importlib.resources.read_text(__package__, 'html.jinja2')
    html = render_template(html_tmpl, context=dict(
        packages=packages,
    ))
    url = target.put_object(
        f'{prefix}.html',
        html.encode('utf8'),
        'text/html',
    )
    return url

def push_ipa(ipa, s3, prefix):
    image_url = None
    icon = ipa.find_best_icon(512)
    if icon is not None:
        with ipa.open_asset(icon.name) as fp:
            image_url = s3.put_object(f'{prefix}.png', fp, 'image/png')

    with open(ipa.filename, 'rb') as fp:
        ipa_url = s3.put_object(f'{prefix}.ipa', fp, 'application/zip')

    manifest_tmpl = importlib.resources.read_text(
        __package__, 'ipa_manifest.jinja2')
    manifest = render_template(manifest_tmpl, context=dict(
        ipa=ipa,
        ipa_url=ipa_url,
        image_url=image_url,
    ))
    manifest_url = s3.put_object(
        f'{prefix}.plist',
        manifest.encode('utf8'),
        'text/xml',
    )

    return dict(
        package_type='ipa',
        app_id=ipa.id,
        display_name=ipa.display_name,
        version_name=ipa.short_version,
        version_code=ipa.version,
        minimum_os_version=ipa.minimum_os_version,
        image_url=image_url,
        ipa_url=ipa_url,
        manifest_url=manifest_url,
    )

def push_apk(apk, s3, prefix):
    image_url = None
    icon = apk.get_app_icon(512)
    if icon and not icon.endswith('.xml'):
        image_data = apk.get_file(icon)
        image_ext = os.path.splitext(icon)[1]
        image_type, _ = mimetypes.guess_type(icon, strict=False)
        image_url = s3.put_object(f'{prefix}{image_ext}', image_data, image_type)

    with open(apk.filename, 'rb') as fp:
        apk_url = s3.put_object(
            f'{prefix}.apk', fp, 'application/vnd.android.package-archive')

    return dict(
        package_type='apk',
        app_id=apk.package,
        display_name=apk.application,
        version_name=apk.version_name,
        version_code=apk.version_code,
        image_url=image_url,
        apk_url=apk_url,
    )



import attr
import os
from urllib.parse import urljoin

log = __import__('logging').getLogger(__name__)

@attr.s(auto_attribs=True)
class LocalTarget:
    url: str
    root_dir: str

    def put_object(self, path, data, content_type):
        url = urljoin(self.url, path)

        path = os.path.join(self.root_dir, path)
        root_dir = os.path.dirname(path)

        os.makedirs(root_dir, exist_ok=True)

        log.info(f'uploading object to {path}')
        with open(path, 'wb') as fp:
            if not isinstance(data, bytes):
                data = data.read()
            fp.write(data)

        return url
